from Song import Song
from tkinter import *
from tkinter import ttk
from matplotlib.figure import Figure
from matplotlib.backends.backend_tkagg import FigureCanvasTkAgg

import matplotlib
import seaborn as sns
import matplotlib.pyplot as plt
import tkinter as tk
import threading

matplotlib.use('TkAgg')


class SongUI(tk.Tk):
    """ UI for Song appplication """

    def __init__(self, song: Song):
        super().__init__()
        self.song = song
        self.init_components()
        self.state = self.show_top_track_spotify
        self.plot()

    def init_components(self):
        self.title('Spotracker')

        self.canvas = FigureCanvasTkAgg(Figure(figsize=(10, 6)), master=self)
        self.ax = self.canvas.figure.add_subplot()
        self.canvas.get_tk_widget().grid(row=3, column=0, rowspan=3, columnspan=4, padx=15, pady=15)

        # Labels
        self.lbl_song = tk.Label(self, text='Select type of rank')
        self.lbl_song.grid(row=0, column=3)

        self.lal_pg = tk.Label(self, text='Progress bar')
        self.lal_pg.grid(row=8, column=0, columnspan=4)

        # Radio buttons
        self.radio = tk.StringVar()
        self.radio1_rd = ttk.Radiobutton(self, text='Spotify', variable=self.radio, value='Spotify', command=self.set_combobox_value)
        self.radio1_rd.grid(row=1, column=0)

        self.radio2_rd = ttk.Radiobutton(self, text='Youtube', variable=self.radio, value='Youtube', command=self.set_combobox_value)
        self.radio2_rd.grid(row=1, column=1)

        self.radio3_rd = ttk.Radiobutton(self, text='Spotify vs Youtube', variable=self.radio, value='Spotify vs Youtube', command=self.set_combobox_value)
        self.radio3_rd.grid(row=1, column=2)
        self.radio.set('Spotify')


        # Combobox
        self.combo = tk.StringVar()
        self.combo_cb = ttk.Combobox(self, textvariable=self.combo, values=list(), state='readonly')
        self.combo_cb.bind('<<ComboboxSelected>>', self.state_combobox)
        self.combo_cb.grid(row=1, column=3)
        self.combo.set('Top 10 Tracks')

        # Listbox
        self.spotify_dict = {'Top 10 Tracks': self.show_top_track_spotify, 'Top 10 Artists': self.show_top_artist_spotify, 'Top 10 Albums': self.show_top_album_spotify}
        self.youtube_dict = {'Top 10 Titles': self.show_top_title_youtube, 'Top 10 Chanel': self.show_top_chanel_youtube, 'Top 10 Albums': self.show_top_album_youtube}
        self.spotify_vs_youtube_dict = {'Top 10 Song & Video': self.show_top_song_spotify_vs_youtube, 'Top 10 Artists & Channel': self.show_top_artist_spotify_vs_youtube, 'Top 10 Albums': self.show_top_album_spotify_vs_youtube}

        # Buttons
        self.reset_btn = tk.Button(self, text='Reset', command=self.reset_handler, bg='green')
        self.reset_btn.grid(row=10, column=0, sticky='w', padx=5, pady=5, rowspan=1)

        self.exit_btn = tk.Button(self, text='Exit', command=self.exit_handler, bg='red')
        self.exit_btn.grid(row=10, column=3, sticky='e', padx=5, pady=5, rowspan=1)

        # Progress bar
        self.progress = ttk.Progressbar(self, orient='horizontal', length=300, mode='indeterminate')
        self.progress.grid(row=9, column=0, columnspan=4)

        self.set_combobox_value()


    # def control_pg(self, data):
    def control_pg(self):
        """ control progress bar """
        self.progress.start()
        self.after(2000, self.check_progress)

    def check_progress(self):
        self.progress.stop()
        self.canvas.draw()


    @property
    def combobox_value(self):
        return self.combo_cb.get()

    def show_top_artist_spotify(self):
        """ show top 10 artist with most streams """
        self.ax.clear()
        graph = sns.barplot(x=self.song.spotify.data['Stream'][:10], y=self.song.spotify.get_artist()[:10], ax=self.ax)
        graph.set_title('Top 10 Artist with most streams')
        graph.set_xlabel('Streams')
        graph.set_ylabel('Artist')
        self.ax.tick_params(axis='y', rotation=0, labelsize=8)



    def show_top_chanel_youtube(self):
        """ show top 10 artist with most views """
        self.ax.clear()
        graph = sns.barplot(x=self.song.youtube.data['Views'][:10], y=self.song.youtube.get_channel()[:10], ax=self.ax)
        graph.set_title('Top 10 Chanel with most views')
        graph.set_xlabel('Views')
        graph.set_ylabel('Chanel')
        self.ax.tick_params(axis='y', rotation=0, labelsize=8)

        

    def show_top_track_spotify(self):
        """ show top 10 track with most streams """
        self.ax.clear()
        graph = sns.barplot(x=self.song.spotify.data['Stream'][:10], y=self.song.spotify.get_track()[:10], ax=self.ax)
        graph.set_title('Top 10 Track with most streams')
        graph.set_xlabel('Streams')
        graph.set_ylabel('Track')
        self.ax.tick_params(axis='y', rotation=0, labelsize=8)


    def show_top_title_youtube(self):
        """ show top 10 title with most views """
        self.ax.clear()
        graph = sns.barplot(x=self.song.youtube.data['Views'][:10], y=self.song.youtube.get_title()[:10], ax=self.ax)
        graph.set_title('Top 10 Title with most views')
        graph.set_xlabel('Views')
        graph.set_ylabel('Title')
        self.ax.tick_params(axis='y', rotation=0, labelsize=8)


    def show_top_album_spotify(self):
        """ show top 10 album with most streams """
        self.ax.clear()
        print("spottify executeddd yayyy")
        graph = sns.barplot(x=self.song.spotify.data['Stream'][:10], y=self.song.spotify.get_album()[:10], ax=self.ax)
        graph.set_title('Top 10 Album with most streams')
        graph.set_xlabel('Streams')
        graph.set_ylabel('Album')
        self.ax.tick_params(axis='y', rotation=0, labelsize=8)


    def show_top_album_youtube(self):
        """ show top 10 album with most views """
        self.ax.clear()
        print("youtube executeddd yayyy")
        graph = sns.barplot(x=self.song.youtube.data['Views'][:10], y=self.song.youtube.get_album()[:10], ax=self.ax)
        graph.set_title('Top 10 Album with most views')
        graph.set_xlabel('Views')
        graph.set_ylabel('Album')
        self.ax.tick_params(axis='y', rotation=0, labelsize=8)


    def show_top_song_spotify_vs_youtube(self):
        """ show top 10 song with most streams and views in one graph """
        self.ax.clear()
        palette_dict = {'Spotify': 'blue', 'Youtube': 'orange'}
        graph = sns.scatterplot(x=self.song.spotify.get_track()[:10], y=self.song.spotify.data['Stream'][:10], ax=self.ax)
        graph = sns.scatterplot(x=self.song.youtube.get_title()[:10], y=self.song.youtube.data['Views'][:10], ax=self.ax)
        graph.set_title('Top 10 Song with most streams and views')
        graph.set_xlabel('Song')
        graph.set_ylabel('Streams and Views')
        self.ax.tick_params(axis='x', rotation=30, labelsize=8)
        self.ax.legend(['Spotify', 'Youtube'])
        self.ax.grid(True)


    def show_top_artist_spotify_vs_youtube(self):
        """ show top 10 artist with most streams and views in line graph """
        self.ax.clear()
        graph = sns.scatterplot(x=self.song.spotify.get_artist()[:10], y=self.song.spotify.data['Stream'][:10], ax=self.ax)
        graph = sns.scatterplot(x=self.song.youtube.get_channel()[:10], y=self.song.youtube.data['Views'][:10], ax=self.ax)
        graph.set_title('Top 10 Artist & Channel with most streams and views')
        graph.set_xlabel('Artist & Channel')
        graph.set_ylabel('Streams and Views')
        self.ax.tick_params(axis='x', rotation=30, labelsize=8)
        self.ax.legend(['Spotify', 'Youtube'])
        self.ax.grid(True)


    def show_top_album_spotify_vs_youtube(self):
        """ show top 10 album with most streams and views in one graph """
        self.ax.clear()
        graph = sns.scatterplot(x=self.song.spotify.get_album()[:10], y=self.song.spotify.data['Stream'][:10], ax=self.ax)
        graph = sns.scatterplot(x=self.song.youtube.get_album()[:10], y=self.song.youtube.data['Views'][:10], ax=self.ax)
        graph.set_title('Top 10 Albums & Channel with most streams and views')
        graph.set_xlabel('Albums & Channel')
        graph.set_ylabel('Streams and Views')
        self.ax.tick_params(axis='x', rotation=30, labelsize=8)
        self.ax.legend(['Spotify', 'Youtube'])
        self.ax.grid(True)


    def set_combobox_value(self):
        """ change combobox values based on radio button selection """
        if self.radio.get() == 'Spotify':
            self.combo_cb.config(state='readonly')
            self.combo_cb.config(values=list(self.spotify_dict.keys()))
            self.combo_cb.set('')
        elif self.radio.get() == 'Youtube':
            self.combo_cb.config(state='readonly')
            self.combo_cb.config(values=list(self.youtube_dict.keys()))
            self.combo_cb.set('')
        elif self.radio.get() == 'Spotify vs Youtube':
            self.combo_cb.set('')
            self.combo_cb.config(state='readonly')
            self.combo_cb.config(values=list(self.spotify_vs_youtube_dict.keys()))
            self.combo_cb.set('')

    def state_combobox(self, events):
        """ change state of combobox based on radio button selection """
        if self.radio.get() == 'Spotify':
            if self.combobox_value == 'Top 10 Tracks':
                self.state = self.show_top_track_spotify
            elif self.combobox_value == 'Top 10 Artists':
                self.state = self.show_top_artist_spotify
            elif self.combobox_value == 'Top 10 Albums':
                self.state = self.show_top_album_spotify
        elif self.radio.get() == 'Youtube':
            if self.combobox_value == 'Top 10 Chanel':
                self.state = self.show_top_chanel_youtube
            elif self.combobox_value == 'Top 10 Titles':
                self.state = self.show_top_title_youtube
            elif self.combobox_value == 'Top 10 Albums':
                self.state = self.show_top_album_youtube
        elif self.radio.get() == 'Spotify vs Youtube':
            if self.combobox_value == 'Top 10 Song & Video':
                self.state = self.show_top_song_spotify_vs_youtube
            elif self.combobox_value == 'Top 10 Artists & Channel':
                self.state = self.show_top_artist_spotify_vs_youtube
            elif self.combobox_value == 'Top 10 Albums':
                self.state = self.show_top_album_spotify_vs_youtube
        self.plot()

    def plot(self):
        """ plot graph """
        self.state()
        self.control_pg()
            
    def reset_handler(self):
        """ reset plot """
        self.combo_cb.set('')
        self.ax.clear()
        self.radio.set('')
        self.canvas.draw()

    def exit_handler(self):
        """ exit program """
        self.destroy()
